/* par_stack.c */
/*
   Copyright (C) 2006 John W. Jordan
   All Rights Reserved

  This file is part of Dante.
  Dante is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the license, or
  ( at your option ) any later version.
  Dante is distributed in the hope that it will be usefull,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A OARTICULAR PURPOSE. See the
  GNU General Public License for more detaile.
  You should have received a copy of the GNU General Public License
  along with Dante; if not write to the Free Software
  Foundation. Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
USA
*/

#include <stdio.h>
#include <malloc.h>
#include "par_stack.h"
#define FRI *(int cmp ())
/* --------------------------------------------------------- */
STACK *
CreateStack (int max, int delta)
{
  STACK *stack;

  stack = (STACK *) xmalloc (sizeof (STACK));
  stack->item = (void **) xmalloc (sizeof (void *) * max);
  stack->max = max;
  stack->delta = delta;
  stack->first = 0;
  stack->last = 0;
  stack->no = 0;
  return (void *) stack;
}

/* --------------------------------------------------------- */
/* Add to the bottom of the stack. */
void
Push (STACK * stack, void *x)
{
  int size;
  if (stack->last == stack->max)
    {
      stack->max += stack->delta;
      size = sizeof (void *) * stack->max;
      stack->item = (void *) realloc (stack->item, size);
      printf ("Increase stack max to %d\n", stack->max);
    }
  stack->no++;
  stack->item[stack->last++] = x;
}

/* --------------------------------------------------------- */
/* Remove from the bottom of the stack. */
void *
Pop (STACK * stack)
{
  if (stack->no == 0)
    {
      stack->first = 0;
      stack->last = 0;
      return 0;
    }
  stack->no--;
  return stack->item[--stack->last];
}

/* --------------------------------------------------------- */
/* Get the k-th item in the stack. */
void *
Item (STACK * stack, int k)
{
  if ((k < 0) || (k >= stack->no))
    return 0;
  return (void *) stack->item[k];
}

/* --------------------------------------------------------- */
/* Next returns the k th item then increments k. */
/* Note that k must be in an array. */
void *
Next (STACK * stack, int k[])
{
  if (k[0] == stack->no)
    return 0;

  return (void *) stack->item[k[0]++];
}

/* --------------------------------------------------------- */
/* Find according to the the cmp function. */
void *
Find_sorted (STACK * stack, void *item, int *(cmp) ())
{
  int i = 0;
  int size;
  int j;
  int k;

  size = Size (stack);

  if (size == 0)
    return 0;

  j = size - 1;

  /* Binary search. */
  while (i <= j)
    {
      k = (i + j) >> 1;
      switch (*(cmp) (item, stack->item[k]))
	{
	case -1:
	  j = k - 1;
	  break;
	case 0:
	  return stack->item[k];
	  break;
	case 1:
	  i = k + 1;
	  break;
	}
    }
  return 0;
}

/* --------------------------------------------------------- */
/* Number of stack items. */
int
Size (STACK * stack)
{
  return stack->no;
}
