/* par_sym_stack.c */
/*
   Copyright (C) 2006 John W. Jordan
   All Rights Reserved

  This file is part of Dante.
  Dante is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the license, or
  ( at your option ) any later version.
  Dante is distributed in the hope that it will be usefull,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A OARTICULAR PURPOSE. See the
  GNU General Public License for more detaile.
  You should have received a copy of the GNU General Public License
  along with Dante; if not write to the Free Software
  Foundation. Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
USA
*/

/* Uses a stack as a symbol table. */
#include <stdio.h>
#include <string.h>
#include "par_stack.h"
#include "par_sym.h"

/* --------------------------------------------------------- */
/* Add sorted sym to stack. */
/*
   Adding to the stack in order requires
   moving data since the stack has the form
   of an array.
*/
SYM *
sym_add (STACK * stack, char *name)
{
  SYM *sym;
  SYM *e;
  SYM *x;
  int k;
  int j;

  if (sym = sym_find_symbol (stack, name))
    return sym;

  /* Default to reserved symbol. */
  sym = (SYM *) xmalloc (sizeof (SYM));
  sym->type = -1;
  sym->name = strdup (name);
  sym->reserved = 0;
  sym->flex_def = 0;

  /* Push onto stack to asssure there is room. */
  /* If stack was empty, just return. */

  Push (stack, sym);

  /* Put in sorted position. */
  x = sym;
  for (k = 0; k < stack->no; k++)
    {
      e = stack->item[k];
      if (strcmp (x->name, e->name) < 0)
	break;
    }
  /* Move others down in array. */
  for (j = stack->no - 1; j > k; j--)
    stack->item[j] = stack->item[j - 1];

  /* Insert */
  stack->item[k] = x;

  return sym;
}

/* ------------------------------------------------ */
char *
sym_name (SYM * sym)
{
  return sym->name;
}

/* ------------------------------------------------ */
int
sym_type_code (SYM * sym)
{
  return sym->type;
}

/* ------------------------------------------------ */
void
sym_set_type (SYM * sym, int type)
{
  sym->type = type;
}

/* ------------------------------------------------ */
char *
sym_set_res (SYM * sym, int res)
{
  sym->reserved = res;
}
/* ------------------------------------------------ */
SYM *
sym_item (STACK *stack, int k)
{ 
  if ((k < 0) || (k >= stack->no))
    return 0;        
  return (SYM *)stack->item[k];
}
/* ------------------------------------------------ */
int
sym_table_size (STACK *stack)
{         
  return stack->no;
}
/* ------------------------------------------------ */
SYM *
sym_next (STACK *stack, int k[1])
{ 
  if ((k[0] < 0) || (k[0] >= stack->no))
    return 0;        
  return (SYM *)stack->item[k[0]++];
}

/* ------------------------------------------------ */
void
sym_add_flexdef (STACK * stack, char *name, char *flex_def)
{
  SYM *sym;
  if ((sym = sym_find_symbol (stack, name)) != 0)
    sym->flex_def = strdup (flex_def);
  else
    {
      fprintf (stderr, "sym_add_flexdef:\n");
      fprintf (stderr, "ERROR:SYMTBL:Name [%s] not in symtbl.\n", name);
      fprintf (stderr, "ERROR:Did not add flex def [%s].\n", flex_def);
    }
}

/* ------------------------------------------------ */
SYM *
sym_find_symbol (STACK * stack, char *name)
{
  SYM *sym;
  int size;
  int i = 0;
  int j;
  int k;

  size = stack->no;

  if (size == 0)
    return 0;

  j = size - 1;

  while (i <= j)
    {
      k = (i + j) >> 1;
      sym = (SYM *) stack->item[k];
      switch (cmpstr (name, sym->name))
	{
	case -1:
	  j = k - 1;
	  break;
	case 0:
	  return sym;
	  break;
	case 1:
	  i = k + 1;
	  break;
	}
    }
  return 0;
}
