/* par_gen_error.c */
/*
   Copyright (C) 2006 John W. Jordan
   All Rights Reserved
*/
/* 
  This file is part of Dante.
  Dante is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the license, or
  ( at your option ) any later version.
  Dante is distributed in the hope that it will be usefull,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A OARTICULAR PURPOSE. See the
  GNU General Public License for more detaile.
  You should have received a copy of the GNU General Public License
  along with Dante; if not write to the Free Software
  Foundation. Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
USA
*/
/* Generates an error procedure. */
#include <stdio.h>
#include <unistd.h>
#include "par.h"
#include "dan_act.h"
/* --------------------------------------------------------- */
static const char *args ="(PAR * par, int errno, char *rule, char *expected)";
static const char *err[7] = {
  "fprintf (stderr, \"ERROR IN APPLYING RULE: %s\\n\", rule);\n",
  "fprintf (stderr, \"ERROR: syntax error number: %d \\n\", errno);\n",
  "fprintf (stderr, \"ERROR: line no:  %d\\n\", par->lineno);\n",
  "fprintf (stderr, \"ERROR: expected: %s\\n\", expected);\n",
  "fprintf (stderr, \"ERROR: got:      %s\\n\", par->symbol);\n",
  "par->error_flag = 1;\n",
  "par->match = 1;\n"
};

/* --------------------------------------------------------- */
void
  par_gen_error (PAR * par) {
  FILE * fd;
  NODE *net;
  NODE *rule;
  NODE *node;
  int k;
  char filename[48];

  sprintf (filename, "par_%s_error.c", par->pname);

  /* If the file exists don`t replace it. */
  if (access (filename, F_OK) == 0)
    {
      if (par->opt_verbose)
	{
	  printf ("%s already exists.\n", filename);
	  printf ("A new %s will not be generated.\n", filename);
	}
      return;
    }

  fd = fopen (filename, "w");

  fprintf (fd, "/* par_%s_error.c */\n", par->pname );
  par_gen_notice (fd);
  fprintf (fd, "#include \"par.h\"\n");
  fprintf (fd, "/* ------------------------------------ */\n");
  fprintf (fd, "int\n");
  fprintf (fd, "par_%s_error %s\n", par->pname, args);
  fprintf (fd, "{\n");

  for (k = 0; k < 7; k++)
      fprintf (fd, "%s", err[k]);
  fprintf (fd, "    switch (errno)\n");
  fprintf (fd, "    {\n");
  
  
  net = (NODE *) par->top_rule;
  for (rule = net->rlink; rule; rule = rule->rlink)
  for (node = rule; node; node = node->nlink) 
	{
	  if ((node->eno) != 0)
	    {
	      fprintf (fd, "\tcase %d: \n", node->eno);
	      fprintf (fd, "\tbreak;\n");
	    }
	}
 
  fprintf (fd, "default:\n");
  fprintf (fd,
	   "\tfprintf (stderr, \"PAR_ERROR: Incorrect error no.\\n\");\n");
  fprintf (fd, "\texit(1);\n");
  fprintf (fd, "\tbreak;\n");
  fprintf (fd, "    } \n"); 
  fprintf (fd, " exit (1);\n");
  fprintf (fd, "} \n");
}
