/* par_gen_lex.c */
/*
   Copyright (C) 2006 John W. Jordan
   All Rights Reserved
*/
/* 
  This file is part of Dante.
  Dante is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the license, or
  ( at your option ) any later version.
  Dante is distributed in the hope that it will be usefull,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A OARTICULAR PURPOSE. See the
  GNU General Public License for more detaile.
  You should have received a copy of the GNU General Public License
  along with Dante; if not write to the Free Software
  Foundation. Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
USA
*/
/* Generates the default Lexer.
   It treats the non-reserved type "identifier" special.
   This is not desirable, but it is hard to define a universal lexer. 
*/

#include "par.h"
#include "dan_act.h"
#include "par_sym.h"

/* ------------- LOCAL PROTOTYPES -------------- */
void par_gen_notice_dan(FILE *fd);
/* --------------------------------------------- */
void
par_gen_lex (PAR * par)
{
  FILE *fd;
  SYM *sym;
  int k;
  int first = 1;
  int is_res;
  int is_iden;

  int int_flag = 0;
  int dbl_flag = 0;
  int num_flag = 0;

  char *name;
  char *pname;
  char filename[48];


  /* Now generate the lexer. */

  pname = par->pname;
  strcpy (filename, "par_");
  strcat (filename, pname);
  strcat (filename, "_lex.dan");

  fd = fopen (filename, "w");
  if (!fd)
    {
      fprintf (stderr,
	       "ERROR:PAR_GEN_LEX:Could not open file for lex code.\n");
      exit (1);
    }
  fprintf (fd, "# par_%s_lex.dan\n", pname);
  par_gen_notice_dan(fd);
  
  fprintf (fd, "{\n");
  fprintf (fd, "#include \"par.h\"\n");
  fprintf (fd, "#include \"par_%s_types.h\"\n", pname);
  fprintf (fd, "}\n");
  fprintf (fd, "%s_lex: @lex_space tokens @print_lexout;\n", pname);

  fprintf (fd, "tokens:\n");

  /* Flag for first token. */
  first = TRUE;

  /* Check for reserved words or "identifier". */
  is_res = FALSE;
  is_iden = FALSE;
  int_flag = 0;
  dbl_flag = 0;
  num_flag = 0;

  k = 0;
  while (sym = (SYM *) Item (par->typetbl, k++))
    {
      is_res = TRUE;
      if (strcmp ("identifier", sym->name) == 0)
	is_iden = TRUE;
      if (strcmp ("integer", sym->name) == 0)
	int_flag = 1;
      if (strcmp ("double", sym->name) == 0)
	dbl_flag = 1;
    }
  if (dbl_flag && int_flag)
    num_flag = 1;

  /* 
     If there are reserved words or identifier add identifier. 
     In either case, compare to reserved words. 
     If there is "identifier" default type is TYPE_identifier.
   */
  if ((is_res) || (is_iden))
    {
      first = FALSE;
      if (is_iden)
	fprintf (fd,
		 "\t  lex_alnum\n\t  { par->type = par_%s_rwtbl (par, TYPE_identifier); }\n",
		 pname);
      else
	{ 
	  fprintf (fd,
		   "\t  lex_alnum\n\t\t{ par->type = par_%s_rwtbl (par, 0); }\n",
		   pname);
	}
    }

  /* All keywords are on the par->typetbl. */
  k = 0;
  while (sym = (SYM *) Item (par->typetbl, k++))
    {
      name = sym->name;

      /* Look for multi-valued types - i.e. not reserved. */
      /* Each will require a lexical function. */
      if (sym->reserved == 0)
	{
	  if (strcmp ("integer", name) == 0)
	    {
	      if (num_flag)
		{
		  fprintf (fd, "\t| lex_number  ");
		  fprintf (fd, "{if ( par->type == 'I')\n");
		  fprintf (fd, "\t\t\tpar->type = TYPE_integer;\n");
		  fprintf (fd, "\t\t\telse par->type = TYPE_double; }\n");
		}
	      else
		fprintf (fd, "\t| lex_%s { par->type = TYPE_%s; }\n",
			 name, name);
	    }
	  else if (strcmp ("double", name) == 0)
	    {
	      if (!num_flag)
		fprintf (fd, "\t| lex_%s { par->type = TYPE_%s; }\n",
			 name, name);
	    }
	  /* Skip identifier. */
	  else if (strcmp ("identifier", name) != 0)
	    {
	      fprintf (fd, "\t| lex_%s { par->type = TYPE_%s; }\n",
		       name, name);
	    }
	}
    }

  fprintf (fd, "\t| lex_operator {par->type = par_%s_optbl(par);}\n", pname);



  fprintf (fd, "\t;\n\n");

  fprintf (fd, "dan_sub_of %s ; \n", par->lname);

  fclose (fd);
}
