/* par_gen_optbl.c */
/*
   Copyright (C) 2006 John W. Jordan
   All Rights Reserved
*/
/* Generates the reserved word table for the lexer. */
#include <stdio.h>
#include <string.h>
#include "par.h"
#include "dan_act.h"
#include "par_sym.h"
/* -------------- PROTOTYPES OF LOCAL PROCEDURES --------------------- */
static void gen_optbl (PAR * par, void *optbl, char *pname);
static void gen_binary_search (PAR * par, FILE * fd);
       void par_gen_notice (FILE *fd);
/* ------------------------------------------------------------------- */
void
par_gen_optbl (PAR * par)
{
  FILE *fd;
  NODE *top;
  SYM *sym;
  void *optbl;
  int k;
  char *pname;

  pname = par->pname;
  optbl = par->littbl;

  /* 
     Generate ...types.h for lexer or regular. But the
     table is not necessary for a lexer. Lexer does direct
     char compares.
   */

  if (par->gen_lexer == FALSE)
    gen_optbl (par, optbl, pname);
}

/* ----------------------------------------------------------------- */
static void
gen_optbl (PAR * par, void *optbl, char *pname)
{
  FILE *fd;
  SYM *sym;
  int k;
  int optbl_size;
  char filename[32];

  /* Construct procedure name: par_xxx_optbl.c */
  strcpy (filename, "par_");
  strcat (filename, pname);
  strcat (filename, "_optbl.c");

  fd = fopen (filename, "w");
  fprintf (fd, "/* par_%s_optbl.c */\n", pname);
  par_gen_notice (fd);
  fprintf (fd, "#include <string.h> \n");
  fprintf (fd, "#include \"par.h\" \n");
  fprintf (fd, "\n");

/* Print the table. */

  optbl_size = sym_table_size (optbl);

  fprintf (fd, "typedef struct \n");
  fprintf (fd, "{ \n");
  fprintf (fd, "    char   *name;\n");
  fprintf (fd, "    int    type;\n");
  fprintf (fd, "}OPTBL; \n\n");
  fprintf (fd, "#define OPTBL_SIZE %d\n", optbl_size);
  fprintf (fd, "static const OPTBL optbl[OPTBL_SIZE] = \n");
  fprintf (fd, "{\n");

  k = 0;
  while (sym = (SYM *) sym_item (optbl, k++))
    fprintf (fd, "   {\"%s\",\t%d },\n", sym->name, sym->type);
  fprintf (fd, "};\n");

  /* Now add the procedural code. */
  gen_binary_search (par, fd);

  fclose (fd);
}
static const char *err1 =
  "\tfprintf (stderr,\n\t\"ERROR: Line number %d\\n\", par->lineno);";
static const char *err2 =
  "\tfprintf (stderr,\n\t\"Invalid operator: [%s]\\n\", par->symbol);";
static const char *err3 =
  "\tfprintf (stderr,\n\t\"Check for missing whitespace between operators.\\n\");";

/* ----------------------------------------------------------------- */
static void
gen_binary_search (PAR * par, FILE * fd)
{
  fprintf (fd, "int par_%s_optbl (PAR *par)\n", par->pname);
  fprintf (fd, "{\n");
  fprintf (fd, "  int i = 0;\n");
  fprintf (fd, "  int j = OPTBL_SIZE - 1;\n");
  fprintf (fd, "  int k;\n\n");

  fprintf (fd, "  /* Test for end of file. */\n");
  fprintf (fd, "  if(par->lex_c == 0) return 0;\n\n");

  fprintf (fd, "  /* Binary search. */\n");
  fprintf (fd, "  while (i <= j)\n");
  fprintf (fd, "    {\n");
  fprintf (fd, "      k = (i + j) >> 1;\n");
  fprintf (fd, "      switch (cmpstr (par->symbol, optbl[k].name))\n");
  fprintf (fd, "      {\n");
  fprintf (fd, "      case -1: j = k - 1; break;\n");
  fprintf (fd,
	   "      case  0: par->type = optbl[k].type; return optbl[k].type; break;\n");
  fprintf (fd, "      case  1: i = k + 1; break;\n");
  fprintf (fd, "      }\n");
  fprintf (fd, "    }\n");

  fprintf (fd, "%s\n", err1);
  fprintf (fd, "%s\n", err2);
  fprintf (fd, "%s\n", err3);
  fprintf (fd, "  exit (1);\n");

  fprintf (fd, "} \n\n");
}
