/* par_gen_par.c */
/*
   Copyright (C) 2006 John W. Jordan
   All Rights Reserved
*/
/* 
  This file is part of Dante.
  Dante is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the license, or
  ( at your option ) any later version.
  Dante is distributed in the hope that it will be usefull,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A OARTICULAR PURPOSE. See the
  GNU General Public License for more detaile.
  You should have received a copy of the GNU General Public License
  along with Dante; if not write to the Free Software
  Foundation. Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
USA
*/
/* Generates a par.h file */
#include <stdio.h>
#include <unistd.h>
#include "par.h"
#include "dan_act.h"

/* ------------------------------------------------------------------- */
static void gen_basic_par (FILE * fd);
static void gen_dante_par (FILE * fd);
static void gen_end_par (FILE * fd);
/* ------------------------------------------------------------------- */
void
par_gen_par (PAR * par)
{
  FILE *fd;
  NODE *node;

  char *name;
  char ename[512];
  char lcname[512];

  /* If the file exists dont replace it. */
  if (access ("par.h", F_OK) == 0)
    {
      if (par->opt_verbose)
	{
	  printf ("Par.h already exists.\n");
	  printf ("Par.h will not be generated.\n");
	}
      return;
    }

  /* Generate par.h */
  /* If pname == dan generate full dante par.h */
  fd = fopen ("par.h", "w");
  gen_basic_par (fd);
  gen_end_par (fd);
  fclose (fd);
}


/* ------------------------------------------------------------------- */
/* Generate basic par.h */
static void
gen_basic_par (FILE * fd)
{
  fprintf (fd, "/* par_.h */\n");
  par_gen_notice (fd);
  fprintf (fd, "/* PROCEDURE GENERATED BY DANTE.*/\n");
  fprintf (fd, "/* DANTE STUB DO NOT EDIT REQUIRED PART. */\n");
  fprintf (fd, "/* MAY BE ADDED TO. */\n");
  fprintf (fd, "#ifndef PAR__H\n");
  fprintf (fd, "#define PAR__H\n");
  fprintf (fd, "#include <stdio.h>\n");
  fprintf (fd, "#include <stdlib.h>\n");
  fprintf (fd, "#include <string.h>\n");
  fprintf (fd, "#include <math.h>\n");

  fprintf (fd, "typedef struct\n");
  fprintf (fd, "{\n");
  fprintf (fd, " char   *symbol;   /* Symbol being assemb. by lexer.*/\n");
  fprintf (fd, " FILE   *fd;       /* Current input file.*/\n");
  fprintf (fd,
	   " void   *act;      /* Pointer to action routine structure.*/\n");
  fprintf (fd, " char   *buffer;   /* buffer for input. */\n");
  fprintf (fd, " double d_value;   /* Return from action routine. */\n");
  fprintf (fd, " char   *s_value;  /* Return from action routine. */\n");
  fprintf (fd, " long   l_value;   /* Return from action routine. */\n");
  fprintf (fd, " int    i_value;   /* Return from action routine. */\n");
  fprintf (fd, " int    sindex;    /* Index into symbol being assembled. */\n");
  fprintf (fd, " int    slimit;    /* Limit of sumbol. */\n");
  fprintf (fd, " int    type;      /* Type returned by lexer. */\n");
  fprintf (fd, " int    req_type;\n");
  fprintf (fd, " int    lex_type;\n");
  fprintf (fd, " int    match;     /* CMP match.*/\n");
  fprintf (fd, " int    lineno;    /* Current line number. */\n");
  fprintf (fd, " int    buf_pos;   /* Char position in input buffer. */\n");
  fprintf (fd, " int    buf_len;   /* Length of buffer contents. */\n");
  fprintf (fd, " int    error_flag;   /* Set if an error is detected. */\n");
  fprintf (fd, " int    buffer_input; /* Flag for buffer input. */\n");
  fprintf (fd, " int    print_input;\n");
  fprintf (fd, " int    print_lexout;\n");

  fprintf (fd, "/* Lexer support.*/\n");
  fprintf (fd, " int lex_c; /* Lex value.*/\n");
  fprintf (fd, " int lex_t; /* Lex type.*/\n");
  fprintf (fd, " int lex_m; /* lex match.*/\n");
  fprintf (fd, " /* End of generic par.h */\n");
}

/* ------------------------------------------------------------------- */
/* Generate end of par.h */
static void
gen_end_par (FILE * fd)
{
  fprintf (fd, "} PAR;\n");
  fprintf (fd, "#endif \n");
}
