/* par_gen_rwtbl.c */
/*
   Copyright (C) 2006 John W. Jordan
   All Rights Reserved
*/
/* 
  This file is part of Dante.
  Dante is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the license, or
  ( at your option ) any later version.
  Dante is distributed in the hope that it will be usefull,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A OARTICULAR PURPOSE. See the
  GNU General Public License for more detaile.
  You should have received a copy of the GNU General Public License
  along with Dante; if not write to the Free Software
  Foundation. Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
USA
*/
/* Generates the reserved word table for the lexer. */
#include <stdio.h>
#include <string.h>
#include "par.h"
#include "dan_act.h"
#include "par_sym.h"
/* -------------- PROTOTYPES OF LOCAL PROCEDURES --------------------- */
static void gen_types_dot_h (PAR * par, void *rwtbl, char *pname);
static void gen_rwtbl (PAR * par, void *rwtbl, char *pname);
static void gen_binary_search (PAR * par, FILE * fd);
/* ------------------------------------------------------------------- */
static const char *err1 =
  "\tfprintf (stderr,\n\t\"ERROR: Search of reserved word table failed.\\n\");";

static const char *err2 =
  "\tfprintf (stderr,\n\t\"ERROR: Could not find [%s].\\n\", par->symbol);";
/* ------------------------------------------------------------------- */
void
par_gen_rwtbl (PAR * par)
{
  FILE *fd;
  NODE *top;
  SYM *sym;
  void *rwtbl = 0;
  char *pname = 0;
  int k;

  pname = par->pname;
  if (pname == 0)
    {
      printf ("PAR_GEN_RWTBL: pname null.\n");
      exit (1);
    }
  rwtbl = par->rwtbl;

  /* 
     Generate ...types.h for lexer or regular. But the
     table is not necessary for a lexer. Lexer does direct
     char compares.
   */

  par_gen_types_h (par);


  if (par->gen_lexer == FALSE)
    gen_rwtbl (par, rwtbl, pname);

}

/* ----------------------------------------------------------------- */
static void
gen_rwtbl (PAR * par, void *rwtbl, char *pname)
{
  FILE *fd;
  SYM *sym;
  int k;
  int rwtbl_size;
  char filename[32];

  /* Construct procedure name: par_xxx_rwtbl.c */
  strcpy (filename, "par_");
  strcat (filename, pname);
  strcat (filename, "_rwtbl.c");

  fd = fopen (filename, "w");
  fprintf (fd, "/* par_%s_rwtbl.c */\n", pname);
  par_gen_notice (fd);
  
  fprintf (fd, "#include <string.h> \n");
  fprintf (fd, "#include \"par.h\" \n");
  fprintf (fd, "#include \"par_%s_types.h\" \n", pname);
  fprintf (fd, "\n");

/* Print the table - rwtbl libary routine. */

  rwtbl_size = sym_table_size (rwtbl);

  fprintf (fd, "typedef struct \n");
  fprintf (fd, "{ \n");
  fprintf (fd, "    char   *name;\n");
  fprintf (fd, "    int    type;\n");
  fprintf (fd, "}RWTBL; \n\n");
  fprintf (fd, "#define RWTBL_SIZE %d\n", rwtbl_size);
  fprintf (fd, "static const RWTBL rwtbl[RWTBL_SIZE] = \n");
  fprintf (fd, "{\n");

  k = 0;
  while (sym = (SYM *) sym_item (rwtbl, k++))
    fprintf (fd, "   {\"%s\",\t%d },\n", sym->name, sym->type);

  fprintf (fd, "};\n");

  /* Now add the procedural code. */
  gen_binary_search (par, fd);

  fclose (fd);
}

/* ----------------------------------------------------------------- */
static void
gen_binary_search (PAR * par, FILE * fd)
{
  fprintf (fd, "int par_%s_rwtbl (PAR *par, int default_type)\n", par->pname);
  fprintf (fd, "{\n");
  fprintf (fd, "  int i = 0;\n");
  fprintf (fd, "  int j = RWTBL_SIZE - 1;\n");
  fprintf (fd, "  int k;\n\n");
  fprintf (fd, "  /* Binary search. */\n");
  fprintf (fd, "  while (i <= j)\n");
  fprintf (fd, "    {\n");
  fprintf (fd, "      k = (i + j) >> 1;\n");
  fprintf (fd, "      switch (cmpstr (par->symbol, rwtbl[k].name))\n");
  fprintf (fd, "      {\n");
  fprintf (fd, "      case -1: j = k - 1; break;\n");
  fprintf (fd,
	   "      case  0: par->type = rwtbl[k].type; return rwtbl[k].type; break;\n");
  fprintf (fd, "      case  1: i = k + 1; break;\n");
  fprintf (fd, "      }\n");
  fprintf (fd, "    }\n");
  fprintf (fd, "  if (default_type == 0)\n");
  fprintf (fd, "  {\n");
  fprintf (fd, "%s      \n", err1);
  fprintf (fd, "%s      \n", err2);
  fprintf (fd, "  }\n");
  fprintf (fd, "  return default_type;\n");
  fprintf (fd, "} \n\n");
}
